@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -38,6 +38,7 @@
 #include <private/qqmlcustomparser_p.h>
 #include <private/qhashedstring_p.h>
 #include <private/qqmlimport_p.h>
+#include <private/qqmlcompiler_p.h>
 
 #include <QtCore/qdebug.h>
 #include <QtCore/qstringlist.h>
@@ -127,7 +128,7 @@ public:
 };
 
 Q_GLOBAL_STATIC(QQmlMetaTypeData, metaTypeData)
-Q_GLOBAL_STATIC_WITH_ARGS(QReadWriteLock, metaTypeDataLock, (QReadWriteLock::Recursive))
+Q_GLOBAL_STATIC_WITH_ARGS(QMutex, metaTypeDataLock, (QMutex::Recursive))
 
 static uint qHash(const QQmlMetaTypeData::VersionedUri &v)
 {
@@ -143,8 +144,7 @@ QQmlMetaTypeData::~QQmlMetaTypeData()
     for (int i = 0; i < types.count(); ++i)
         delete types.at(i);
 
-    TypeModules::const_iterator i = uriToModule.constBegin();
-    for (; i != uriToModule.constEnd(); ++i)
+    for (TypeModules::const_iterator i = uriToModule.constBegin(), cend = uriToModule.constEnd(); i != cend; ++i)
         delete *i;
 }
 
@@ -218,24 +218,23 @@ public:
 void QQmlType::SingletonInstanceInfo::init(QQmlEngine *e)
 {
     QV4::ExecutionEngine *v4 = QV8Engine::getV4(e->handle());
+    v4->pushGlobalContext();
     if (scriptCallback && scriptApi(e).isUndefined()) {
-        v4->pushGlobalContext();
         setScriptApi(e, scriptCallback(e, e));
-        v4->popContext();
     } else if (qobjectCallback && !qobjectApi(e)) {
-        v4->pushGlobalContext();
         QObject *o = qobjectCallback(e, e);
         setQObjectApi(e, o);
+        if (!o) {
+            qFatal("qmlRegisterSingletonType(): \"%s\" is not available because the callback function returns a null pointer.", qPrintable(typeName));
+        }
         // if this object can use a property cache, create it now
         QQmlData::ensurePropertyCache(e, o);
-        v4->popContext();
     } else if (!url.isEmpty() && !qobjectApi(e)) {
-        v4->pushGlobalContext();
         QQmlComponent component(e, url, QQmlComponent::PreferSynchronous);
         QObject *o = component.create();
         setQObjectApi(e, o);
-        v4->popContext();
     }
+    v4->popContext();
 }
 
 void QQmlType::SingletonInstanceInfo::destroy(QQmlEngine *e)
@@ -243,7 +242,12 @@ void QQmlType::SingletonInstanceInfo::destroy(QQmlEngine *e)
     // cleans up the engine-specific singleton instances if they exist.
     scriptApis.remove(e);
     QObject *o = qobjectApis.take(e);
-    delete o;
+    if (o) {
+        QQmlData *ddata = QQmlData::get(o, false);
+        if (url.isEmpty() && ddata && ddata->indestructible && ddata->explicitIndestructibleSet)
+            return;
+        delete o;
+    }
 }
 
 void QQmlType::SingletonInstanceInfo::setQObjectApi(QQmlEngine *e, QObject *o)
@@ -256,7 +260,7 @@ QObject *QQmlType::SingletonInstanceInfo::qobjectApi(QQmlEngine *e) const
     return qobjectApis.value(e);
 }
 
-void QQmlType::SingletonInstanceInfo::setScriptApi(QQmlEngine *e, QJSValue v)
+void QQmlType::SingletonInstanceInfo::setScriptApi(QQmlEngine *e, const QJSValue &v)
 {
     scriptApis.insert(e, v);
 }
@@ -475,6 +479,29 @@ QQmlType *QQmlType::superType() const
     return d->superType;
 }
 
+QQmlType *QQmlType::resolveCompositeBaseType(QQmlEnginePrivate *engine) const
+{
+    Q_ASSERT(isComposite());
+    if (!engine)
+        return 0;
+    QQmlTypeData *td = engine->typeLoader.getType(sourceUrl());
+    if (!td || !td->isComplete())
+        return 0;
+    QQmlCompiledData *cd = td->compiledData();
+    const QMetaObject *mo = cd->rootPropertyCache->firstCppMetaObject();
+    return QQmlMetaType::qmlType(mo);
+}
+
+int QQmlType::resolveCompositeEnumValue(QQmlEnginePrivate *engine, const QString &name, bool *ok) const
+{
+    Q_ASSERT(isComposite());
+    *ok = false;
+    QQmlType *type = resolveCompositeBaseType(engine);
+    if (!type)
+        return -1;
+    return type->enumValue(engine, name, ok);
+}
+
 static void clone(QMetaObjectBuilder &builder, const QMetaObject *mo,
                   const QMetaObject *ignoreStart, const QMetaObject *ignoreEnd)
 {
@@ -566,7 +593,7 @@ void QQmlTypePrivate::init() const
     if (isSetup)
         return;
 
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     if (isSetup)
         return;
 
@@ -647,7 +674,7 @@ void QQmlTypePrivate::initEnums() const
 
     init();
 
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     if (isEnumSetup) return;
 
     if (baseMetaObject) // could be singleton type without metaobject
@@ -840,18 +867,26 @@ int QQmlType::metaObjectRevision() const
     return d->revision;
 }
 
-QQmlAttachedPropertiesFunc QQmlType::attachedPropertiesFunction() const
+QQmlAttachedPropertiesFunc QQmlType::attachedPropertiesFunction(QQmlEnginePrivate *engine) const
 {
-    if (d->regType != CppType)
-        return 0;
-    return d->extraData.cd->attachedPropertiesFunc;
+    if (d->regType == CppType)
+        return d->extraData.cd->attachedPropertiesFunc;
+
+    QQmlType *base = 0;
+    if (d->regType == CompositeType)
+        base = resolveCompositeBaseType(engine);
+    return base ? base->attachedPropertiesFunction(engine) : 0;
 }
 
-const QMetaObject *QQmlType::attachedPropertiesType() const
+const QMetaObject *QQmlType::attachedPropertiesType(QQmlEnginePrivate *engine) const
 {
-    if (d->regType != CppType)
-        return 0;
-    return d->extraData.cd->attachedPropertiesType;
+    if (d->regType == CppType)
+        return d->extraData.cd->attachedPropertiesType;
+
+    QQmlType *base = 0;
+    if (d->regType == CompositeType)
+        base = resolveCompositeBaseType(engine);
+    return base ? base->attachedPropertiesType(engine) : 0;
 }
 
 /*
@@ -859,11 +894,15 @@ This is the id passed to qmlAttachedPropertiesById().  This is different from th
 for the case that a single class is registered under two or more names (eg. Item in
 Qt 4.7 and QtQuick 1.0).
 */
-int QQmlType::attachedPropertiesId() const
+int QQmlType::attachedPropertiesId(QQmlEnginePrivate *engine) const
 {
-    if (d->regType != CppType)
-        return 0;
-    return d->extraData.cd->attachedPropertiesId;
+    if (d->regType == CppType)
+        return d->extraData.cd->attachedPropertiesId;
+
+    QQmlType *base = 0;
+    if (d->regType == CompositeType)
+        base = resolveCompositeBaseType(engine);
+    return base ? base->attachedPropertiesId(engine) : 0;
 }
 
 int QQmlType::parserStatusCast() const
@@ -909,9 +948,11 @@ QUrl QQmlType::sourceUrl() const
         return QUrl();
 }
 
-int QQmlType::enumValue(const QHashedStringRef &name, bool *ok) const
+int QQmlType::enumValue(QQmlEnginePrivate *engine, const QHashedStringRef &name, bool *ok) const
 {
     Q_ASSERT(ok);
+    if (isComposite())
+        return resolveCompositeEnumValue(engine, name.toString(), ok);
     *ok = true;
 
     d->initEnums();
@@ -924,9 +965,11 @@ int QQmlType::enumValue(const QHashedStringRef &name, bool *ok) const
     return -1;
 }
 
-int QQmlType::enumValue(const QHashedCStringRef &name, bool *ok) const
+int QQmlType::enumValue(QQmlEnginePrivate *engine, const QHashedCStringRef &name, bool *ok) const
 {
     Q_ASSERT(ok);
+    if (isComposite())
+        return resolveCompositeEnumValue(engine, name.toUtf16(), ok);
     *ok = true;
 
     d->initEnums();
@@ -939,9 +982,11 @@ int QQmlType::enumValue(const QHashedCStringRef &name, bool *ok) const
     return -1;
 }
 
-int QQmlType::enumValue(const QV4::String *name, bool *ok) const
+int QQmlType::enumValue(QQmlEnginePrivate *engine, const QV4::String *name, bool *ok) const
 {
     Q_ASSERT(ok);
+    if (isComposite())
+        return resolveCompositeEnumValue(engine, name->toQString(), ok);
     *ok = true;
 
     d->initEnums();
@@ -1001,7 +1046,7 @@ void QQmlTypeModulePrivate::add(QQmlType *type)
 
 QQmlType *QQmlTypeModule::type(const QHashedStringRef &name, int minor)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
 
     QList<QQmlType *> *types = d->typeHash.value(name);
     if (!types) return 0;
@@ -1015,7 +1060,7 @@ QQmlType *QQmlTypeModule::type(const QHashedStringRef &name, int minor)
 
 QQmlType *QQmlTypeModule::type(const QV4::String *name, int minor)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
 
     QList<QQmlType *> *types = d->typeHash.value(name);
     if (!types) return 0;
@@ -1029,7 +1074,7 @@ QQmlType *QQmlTypeModule::type(const QV4::String *name, int minor)
 
 QList<QQmlType*> QQmlTypeModule::singletonTypes(int minor) const
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
 
     QList<QQmlType *> retn;
     for (int ii = 0; ii < d->types.count(); ++ii) {
@@ -1090,14 +1135,13 @@ QQmlType *QQmlTypeModuleVersion::type(const QV4::String *name) const
 void qmlClearTypeRegistrations() // Declared in qqml.h
 {
     //Only cleans global static, assumed no running engine
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     for (int i = 0; i < data->types.count(); ++i)
         delete data->types.at(i);
 
-    QQmlMetaTypeData::TypeModules::const_iterator i = data->uriToModule.constBegin();
-    for (; i != data->uriToModule.constEnd(); ++i)
+    for (QQmlMetaTypeData::TypeModules::const_iterator i = data->uriToModule.constBegin(), cend = data->uriToModule.constEnd(); i != cend; ++i)
         delete *i;
 
     data->types.clear();
@@ -1109,12 +1153,14 @@ void qmlClearTypeRegistrations() // Declared in qqml.h
     data->uriToModule.clear();
 
     QQmlEnginePrivate::baseModulesUninitialized = true; //So the engine re-registers its types
+#ifndef QT_NO_LIBRARY
     qmlClearEnginePlugins();
+#endif
 }
 
 int registerAutoParentFunction(QQmlPrivate::RegisterAutoParent &autoparent)
 {
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     data->parentFunctions.append(autoparent.function);
@@ -1127,7 +1173,7 @@ int registerInterface(const QQmlPrivate::RegisterInterface &interface)
     if (interface.version > 0)
         qFatal("qmlRegisterType(): Cannot mix incompatible QML versions.");
 
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     int index = data->types.count();
@@ -1165,7 +1211,7 @@ QString registrationTypeString(QQmlType::RegistrationType typeType)
     return typeStr;
 }
 
-// NOTE: caller must hold a QWriteLocker on "data"
+// NOTE: caller must hold a QMutexLocker on "data"
 bool checkRegistration(QQmlType::RegistrationType typeType, QQmlMetaTypeData *data, const char *uri, const QString &typeName, int majorVersion = -1)
 {
     if (!typeName.isEmpty()) {
@@ -1216,7 +1262,7 @@ bool checkRegistration(QQmlType::RegistrationType typeType, QQmlMetaTypeData *da
     return true;
 }
 
-// NOTE: caller must hold a QWriteLocker on "data"
+// NOTE: caller must hold a QMutexLocker on "data"
 void addTypeToData(QQmlType* type, QQmlMetaTypeData *data)
 {
     if (!type->elementName().isEmpty())
@@ -1255,7 +1301,7 @@ void addTypeToData(QQmlType* type, QQmlMetaTypeData *data)
 
 int registerType(const QQmlPrivate::RegisterType &type)
 {
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     QString elementName = QString::fromUtf8(type.elementName);
     if (!checkRegistration(QQmlType::CppType, data, type.uri, elementName, type.versionMajor))
@@ -1275,7 +1321,7 @@ int registerType(const QQmlPrivate::RegisterType &type)
 
 int registerSingletonType(const QQmlPrivate::RegisterSingletonType &type)
 {
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     QString typeName = QString::fromUtf8(type.typeName);
     if (!checkRegistration(QQmlType::SingletonType, data, type.uri, typeName, type.versionMajor))
@@ -1294,7 +1340,7 @@ int registerSingletonType(const QQmlPrivate::RegisterSingletonType &type)
 int registerCompositeSingletonType(const QQmlPrivate::RegisterCompositeSingletonType &type)
 {
     // Assumes URL is absolute and valid. Checking of user input should happen before the URL enters type.
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     QString typeName = QString::fromUtf8(type.typeName);
     bool fileImport = false;
@@ -1319,7 +1365,7 @@ int registerCompositeSingletonType(const QQmlPrivate::RegisterCompositeSingleton
 int registerCompositeType(const QQmlPrivate::RegisterCompositeType &type)
 {
     // Assumes URL is absolute and valid. Checking of user input should happen before the URL enters type.
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     QString typeName = QString::fromUtf8(type.typeName);
     bool fileImport = false;
@@ -1344,7 +1390,7 @@ int registerQmlUnitCacheHook(const QQmlPrivate::RegisterQmlUnitCacheHook &hookRe
 {
     if (hookRegistration.version > 0)
         qFatal("qmlRegisterType(): Cannot mix incompatible QML versions.");
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     data->lookupCachedQmlUnit << hookRegistration.lookupCachedQmlUnit;
     return 0;
@@ -1377,7 +1423,7 @@ int QQmlPrivate::qmlregister(RegistrationType type, void *data)
 //From qqml.h
 bool qmlProtectModule(const char *uri, int majVersion)
 {
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     QQmlMetaTypeData::VersionedUri versionedUri;
@@ -1391,14 +1437,14 @@ bool qmlProtectModule(const char *uri, int majVersion)
     return false;
 }
 
-bool QQmlMetaType::namespaceContainsRegistrations(const QString &uri)
+bool QQmlMetaType::namespaceContainsRegistrations(const QString &uri, int majorVersion)
 {
     QQmlMetaTypeData *data = metaTypeData();
 
     // Has any type previously been installed to this namespace?
     QHashedString nameSpace(uri);
     foreach (const QQmlType *type, data->types)
-        if (type->module() == nameSpace)
+        if (type->module() == nameSpace && type->majorVersion() == majorVersion)
             return true;
 
     return false;
@@ -1426,7 +1472,7 @@ QStringList QQmlMetaType::typeRegistrationFailures()
     return data->typeRegistrationFailures;
 }
 
-QReadWriteLock *QQmlMetaType::typeRegistrationLock()
+QMutex *QQmlMetaType::typeRegistrationLock()
 {
     return metaTypeDataLock();
 }
@@ -1436,11 +1482,11 @@ QReadWriteLock *QQmlMetaType::typeRegistrationLock()
 */
 bool QQmlMetaType::isAnyModule(const QString &uri)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
-    for (QQmlMetaTypeData::TypeModules::ConstIterator iter = data->uriToModule.begin();
-         iter != data->uriToModule.end(); ++iter) {
+    for (QQmlMetaTypeData::TypeModules::ConstIterator iter = data->uriToModule.cbegin();
+         iter != data->uriToModule.cend(); ++iter) {
         if ((*iter)->module() == uri)
             return true;
     }
@@ -1453,7 +1499,7 @@ bool QQmlMetaType::isAnyModule(const QString &uri)
 */
 bool QQmlMetaType::isLockedModule(const QString &uri, int majVersion)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     QQmlMetaTypeData::VersionedUri versionedUri;
@@ -1474,7 +1520,7 @@ bool QQmlMetaType::isLockedModule(const QString &uri, int majVersion)
 bool QQmlMetaType::isModule(const QString &module, int versionMajor, int versionMinor)
 {
     Q_ASSERT(versionMajor >= 0 && versionMinor >= 0);
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
 
     QQmlMetaTypeData *data = metaTypeData();
 
@@ -1489,14 +1535,14 @@ bool QQmlMetaType::isModule(const QString &module, int versionMajor, int version
 
 QQmlTypeModule *QQmlMetaType::typeModule(const QString &uri, int majorVersion)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     return data->uriToModule.value(QQmlMetaTypeData::VersionedUri(uri, majorVersion));
 }
 
 QList<QQmlPrivate::AutoParentFunction> QQmlMetaType::parentFunctions()
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     return data->parentFunctions;
 }
@@ -1510,7 +1556,7 @@ QObject *QQmlMetaType::toQObject(const QVariant &v, bool *ok)
 
     if (ok) *ok = true;
 
-    return *(QObject **)v.constData();
+    return *(QObject *const *)v.constData();
 }
 
 bool QQmlMetaType::isQObject(int userType)
@@ -1518,7 +1564,7 @@ bool QQmlMetaType::isQObject(int userType)
     if (userType == QMetaType::QObjectStar)
         return true;
 
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     return userType >= 0 && userType < data->objects.size() && data->objects.testBit(userType);
 }
@@ -1528,7 +1574,7 @@ bool QQmlMetaType::isQObject(int userType)
  */
 int QQmlMetaType::listType(int id)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     QQmlType *type = data->idToType.value(id);
     if (type && type->qListTypeId() == id)
@@ -1537,25 +1583,25 @@ int QQmlMetaType::listType(int id)
         return 0;
 }
 
-int QQmlMetaType::attachedPropertiesFuncId(const QMetaObject *mo)
+int QQmlMetaType::attachedPropertiesFuncId(QQmlEnginePrivate *engine, const QMetaObject *mo)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     QQmlType *type = data->metaObjectToType.value(mo);
-    if (type && type->attachedPropertiesFunction())
-        return type->attachedPropertiesId();
+    if (type && type->attachedPropertiesFunction(engine))
+        return type->attachedPropertiesId(engine);
     else
         return -1;
 }
 
-QQmlAttachedPropertiesFunc QQmlMetaType::attachedPropertiesFuncById(int id)
+QQmlAttachedPropertiesFunc QQmlMetaType::attachedPropertiesFuncById(QQmlEnginePrivate *engine, int id)
 {
     if (id < 0)
         return 0;
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
-    return data->types.at(id)->attachedPropertiesFunction();
+    return data->types.at(id)->attachedPropertiesFunction(engine);
 }
 
 QMetaProperty QQmlMetaType::defaultProperty(const QMetaObject *metaObject)
@@ -1617,7 +1663,7 @@ QQmlMetaType::TypeCategory QQmlMetaType::typeCategory(int userType)
     if (userType == QMetaType::QObjectStar)
         return Object;
 
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     if (userType < data->objects.size() && data->objects.testBit(userType))
         return Object;
@@ -1629,14 +1675,14 @@ QQmlMetaType::TypeCategory QQmlMetaType::typeCategory(int userType)
 
 bool QQmlMetaType::isInterface(int userType)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     return userType >= 0 && userType < data->interfaces.size() && data->interfaces.testBit(userType);
 }
 
 const char *QQmlMetaType::interfaceIId(int userType)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     QQmlType *type = data->idToType.value(userType);
     lock.unlock();
@@ -1648,7 +1694,7 @@ const char *QQmlMetaType::interfaceIId(int userType)
 
 bool QQmlMetaType::isList(int userType)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     return userType >= 0 && userType < data->lists.size() && data->lists.testBit(userType);
 }
@@ -1670,7 +1716,7 @@ bool QQmlMetaType::isList(int userType)
  */
 void QQmlMetaType::registerCustomStringConverter(int type, StringConverter converter)
 {
-    QWriteLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
 
     QQmlMetaTypeData *data = metaTypeData();
     if (data->stringConverters.contains(type))
@@ -1684,7 +1730,7 @@ void QQmlMetaType::registerCustomStringConverter(int type, StringConverter conve
  */
 QQmlMetaType::StringConverter QQmlMetaType::customStringConverter(int type)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
 
     QQmlMetaTypeData *data = metaTypeData();
     return data->stringConverters.value(type);
@@ -1713,11 +1759,11 @@ QQmlType *QQmlMetaType::qmlType(const QString &qualifiedName, int version_major,
 QQmlType *QQmlMetaType::qmlType(const QHashedStringRef &name, const QHashedStringRef &module, int version_major, int version_minor)
 {
     Q_ASSERT(version_major >= 0 && version_minor >= 0);
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     QQmlMetaTypeData::Names::ConstIterator it = data->nameToType.constFind(name);
-    while (it != data->nameToType.end() && it.key() == name) {
+    while (it != data->nameToType.cend() && it.key() == name) {
         // XXX version_major<0 just a kludge for QQmlPropertyPrivate::initProperty
         if (version_major < 0 || module.isEmpty() || (*it)->availableInVersion(module, version_major,version_minor))
             return (*it);
@@ -1733,7 +1779,7 @@ QQmlType *QQmlMetaType::qmlType(const QHashedStringRef &name, const QHashedStrin
 */
 QQmlType *QQmlMetaType::qmlType(const QMetaObject *metaObject)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     return data->metaObjectToType.value(metaObject);
@@ -1747,11 +1793,11 @@ QQmlType *QQmlMetaType::qmlType(const QMetaObject *metaObject)
 QQmlType *QQmlMetaType::qmlType(const QMetaObject *metaObject, const QHashedStringRef &module, int version_major, int version_minor)
 {
     Q_ASSERT(version_major >= 0 && version_minor >= 0);
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     QQmlMetaTypeData::MetaObjects::const_iterator it = data->metaObjectToType.constFind(metaObject);
-    while (it != data->metaObjectToType.end() && it.key() == metaObject) {
+    while (it != data->metaObjectToType.cend() && it.key() == metaObject) {
         QQmlType *t = *it;
         if (version_major < 0 || module.isEmpty() || t->availableInVersion(module, version_major,version_minor))
             return t;
@@ -1767,7 +1813,7 @@ QQmlType *QQmlMetaType::qmlType(const QMetaObject *metaObject, const QHashedStri
 */
 QQmlType *QQmlMetaType::qmlType(int userType)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     QQmlType *type = data->idToType.value(userType);
@@ -1785,7 +1831,7 @@ QQmlType *QQmlMetaType::qmlType(int userType)
 */
 QQmlType *QQmlMetaType::qmlType(const QUrl &url, bool includeNonFileImports /* = false */)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     QQmlType *type = data->urlToType.value(url);
@@ -1805,7 +1851,7 @@ QQmlType *QQmlMetaType::qmlType(const QUrl &url, bool includeNonFileImports /* =
 */
 QQmlType *QQmlMetaType::qmlTypeFromIndex(int idx)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     if (idx < 0 || idx >= data->types.count())
@@ -1818,12 +1864,13 @@ QQmlType *QQmlMetaType::qmlTypeFromIndex(int idx)
 */
 QList<QString> QQmlMetaType::qmlTypeNames()
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     QList<QString> names;
-    QQmlMetaTypeData::Names::ConstIterator it = data->nameToType.begin();
-    while (it != data->nameToType.end()) {
+    names.reserve(data->nameToType.count());
+    QQmlMetaTypeData::Names::ConstIterator it = data->nameToType.cbegin();
+    while (it != data->nameToType.cend()) {
         names += (*it)->qmlTypeName();
         ++it;
     }
@@ -1836,7 +1883,7 @@ QList<QString> QQmlMetaType::qmlTypeNames()
 */
 QList<QQmlType*> QQmlMetaType::qmlTypes()
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     return data->nameToType.values();
@@ -1847,7 +1894,7 @@ QList<QQmlType*> QQmlMetaType::qmlTypes()
 */
 QList<QQmlType*> QQmlMetaType::qmlAllTypes()
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     return data->types;
@@ -1858,7 +1905,7 @@ QList<QQmlType*> QQmlMetaType::qmlAllTypes()
 */
 QList<QQmlType*> QQmlMetaType::qmlSingletonTypes()
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
 
     QList<QQmlType*> alltypes = data->nameToType.values();
@@ -1874,7 +1921,7 @@ QList<QQmlType*> QQmlMetaType::qmlSingletonTypes()
 
 const QQmlPrivate::CachedQmlUnit *QQmlMetaType::findCachedCompilationUnit(const QUrl &uri)
 {
-    QReadLocker lock(metaTypeDataLock());
+    QMutexLocker lock(metaTypeDataLock());
     QQmlMetaTypeData *data = metaTypeData();
     for (QVector<QQmlPrivate::QmlUnitCacheLookupFunction>::ConstIterator it = data->lookupCachedQmlUnit.constBegin(), end = data->lookupCachedQmlUnit.constEnd();
          it != end; ++it) {
@@ -1884,4 +1931,43 @@ const QQmlPrivate::CachedQmlUnit *QQmlMetaType::findCachedCompilationUnit(const
     return 0;
 }
 
+/*!
+    Returns the pretty QML type name (e.g. 'Item' instead of 'QtQuickItem') for the given object.
+ */
+QString QQmlMetaType::prettyTypeName(const QObject *object)
+{
+    QString typeName;
+
+    if (!object)
+        return typeName;
+
+    const QQmlType *type = QQmlMetaType::qmlType(object->metaObject());
+    if (type) {
+        typeName = type->qmlTypeName();
+        const int lastSlash = typeName.lastIndexOf(QLatin1Char('/'));
+        if (lastSlash != -1)
+            typeName = typeName.mid(lastSlash + 1);
+    } else {
+        typeName = QString::fromUtf8(object->metaObject()->className());
+        int marker = typeName.indexOf(QLatin1String("_QMLTYPE_"));
+        if (marker != -1)
+            typeName = typeName.left(marker);
+
+        marker = typeName.indexOf(QLatin1String("_QML_"));
+        if (marker != -1) {
+            typeName = typeName.left(marker);
+            typeName += QLatin1Char('*');
+            type = QQmlMetaType::qmlType(QMetaType::type(typeName.toLatin1()));
+            if (type) {
+                typeName = type->qmlTypeName();
+                const int lastSlash = typeName.lastIndexOf(QLatin1Char('/'));
+                if (lastSlash != -1)
+                    typeName = typeName.mid(lastSlash + 1);
+            }
+        }
+    }
+
+    return typeName;
+}
+
 QT_END_NAMESPACE
